REM ---------------------------------------------------------------------------
REM  Trivadis - http://www.trivadis.com
REM ---------------------------------------------------------------------------
REM  Group/Privileges.: User
REM  Script-Name......: ex_bind_peeking.sql
REM  Developer........: Christian Antognini (christian.antognini@trivadis.com)
REM  Date.............: November 2007
REM  Version..........: Oracle Database 11g Release 1 and Release 2
REM  Description......: The script shows that the database engine automatically
REM                     recognize that an execution plan is not efficient for
REM                     all bind variable values.
REM  Usage............: 
REM                     
REM  Input parameters.: 
REM  Output...........: 
REM  Called by........:
REM  Requirements.....: 
REM  Remarks..........: Before re-executing the script the shared pool 
REM                     should be flushed.
REM ---------------------------------------------------------------------------
REM Changes:
REM DD.MM.YYYY Developer Change
REM ---------------------------------------------------------------------------
REM 12.12.2011 ChA       Changed header, columns definition and added cleanup
REM ---------------------------------------------------------------------------

SET ECHO ON TERMOUT ON SERVEROUTPUT OFF FEEDBACK OFF PAGESIZE 100 LINESIZE 80

CLEAR COLUMNS
COLUMN PLAN_TABLE_OUTPUT FORMAT A70
COLUMN PREDICATE FORMAT A20
COLUMN EXECUTIONS FORMAT 9999 HEADING EXEC
COLUMN PEEKED FORMAT A6
COLUMN ROWS_PROCESSED FORMAT 9999 HEADING ROWS
COLUMN BUFFER_GETS FORMAT 9999 HEADING GETS
COLUMN CPU_TIME FORMAT 9999 HEADING CPU
COLUMN SQL_ID NEW_VALUE SQL_ID
COLUMN IS_BIND_SENSITIVE FORMAT A17
COLUMN IS_BIND_AWARE FORMAT A13
COLUMN IS_SHAREABLE FORMAT A12

DROP TABLE t PURGE;

VARIABLE n NUMBER

CLEAR SCREEN

REM
REM create test table and gather statistics
REM

CREATE TABLE t (id, n, pad, CONSTRAINT t_pk PRIMARY KEY (id))
AS 
SELECT rownum, CASE WHEN rownum>500 THEN 666 ELSE rownum END, rpad('*',50,'*') 
FROM dba_objects WHERE rownum <= 1000;

CREATE INDEX i ON t (n);

BEGIN
  dbms_stats.gather_table_stats(
    ownname => user, 
    tabname => 't', 
    estimate_percent => 100, 
    method_opt => 'for all columns size 254',
    cascade => TRUE
  );
END;
/

PAUSE

REM
REM show execution plan generated when peeked value is selective
REM

execute :n := 6

SELECT count(pad) FROM t WHERE n = :n;

SELECT * FROM table(dbms_xplan.display_cursor(NULL, NULL, 'basic'));

PAUSE

execute :n := 19

SELECT count(pad) FROM t WHERE n = :n;

SELECT * FROM table(dbms_xplan.display_cursor(NULL, NULL, 'basic'));

PAUSE

execute :n := 42

SELECT count(pad) FROM t WHERE n = :n;

SELECT * FROM table(dbms_xplan.display_cursor(NULL, NULL, 'basic'));

PAUSE

REM
REM show execution plan generated when peeked not selective,
REM notice that the previous plan is reused... but only
REM for 3 executions...
REM

execute :n := 666

SELECT count(pad) FROM t WHERE n = :n;

SELECT * FROM table(dbms_xplan.display_cursor(NULL, NULL, 'basic'));

PAUSE

execute :n := 666

SELECT count(pad) FROM t WHERE n = :n;

SELECT * FROM table(dbms_xplan.display_cursor(NULL, NULL, 'basic'));

PAUSE

execute :n := 666

SELECT count(pad) FROM t WHERE n = :n;

SELECT * FROM table(dbms_xplan.display_cursor(NULL, NULL, 'basic'));

PAUSE

execute :n := 666

SELECT count(pad) FROM t WHERE n = :n;

SELECT * FROM table(dbms_xplan.display_cursor(NULL, NULL, 'basic'));

PAUSE

REM
REM if a selective value is used a different plan is used
REM

execute :n := 42

SELECT count(pad) FROM t WHERE n = :n;

SELECT * FROM table(dbms_xplan.display_cursor(NULL, NULL, 'basic'));

PAUSE

REM
REM find out sql_id
REM

SELECT count(pad) FROM t WHERE n = :n;

SELECT prev_sql_id AS sql_id
FROM v$session
WHERE sid = sys_context('userenv','sid');

PAUSE

REM
REM let's have a look to the shared pool
REM

SELECT child_number, is_bind_sensitive, is_bind_aware, is_shareable
FROM v$sql
WHERE sql_id = '&sql_id';

PAUSE

SELECT child_number, predicate, range_id, low, high
FROM v$sql_cs_selectivity 
WHERE sql_id = '&sql_id'
ORDER BY child_number;

PAUSE

SELECT child_number, bucket_id, count
FROM v$sql_cs_histogram 
WHERE sql_id = '&sql_id'
ORDER BY child_number, bucket_id;

PAUSE

SELECT child_number, bind_set_hash_value, peeked, executions, rows_processed, buffer_gets, cpu_time
FROM v$sql_cs_statistics 
WHERE sql_id = '&sql_id'
ORDER BY child_number;

PAUSE

REM
REM Cleanup
REM

CLEAR COLUMNS
DROP TABLE t PURGE;
