REM ---------------------------------------------------------------------------
REM  Trivadis - http://www.trivadis.com
REM ---------------------------------------------------------------------------
REM  Group/Privileges.: User
REM  Script-Name......: ex_cardinality_feedback.sql
REM  Developer........: Christian Antognini (christian.antognini@trivadis.com)
REM  Date.............: December 2011
REM  Version..........: Oracle Database 11g Release 2
REM  Description......: The script demonstrates the feature called 
REM                     "Cardinality Feedback".
REM  Usage............: 
REM                     
REM  Input parameters.: 
REM  Output...........: 
REM  Called by........:
REM  Requirements.....: 
REM  Remarks..........: Before re-executing the script the shared pool 
REM                     should be flushed.
REM ---------------------------------------------------------------------------
REM Changes:
REM DD.MM.YYYY Developer Change
REM ---------------------------------------------------------------------------
REM 12.12.2011 ChA       Changed header, columns definition and added cleanup
REM ---------------------------------------------------------------------------

SET ECHO ON TERMOUT ON SERVEROUTPUT OFF FEEDBACK OFF PAGESIZE 100 LINESIZE 120

DROP TABLE t PURGE;
DROP FUNCTION f;
DROP TYPE number_t;

VARIABLE n NUMBER

COLUMN pad FORMAT A10 TRUNC
COLUMN is_shareable FORMAT A12
COLUMN use_feedback_stats FORMAT A18
COLUMN sql_id NEW_VALUE sql_id

CLEAR SCREEN

REM
REM Setup environment
REM

CREATE TABLE t AS SELECT rownum AS n, rpad('*',100,'*') AS pad FROM dual CONNECT BY level <= 10000;

CREATE INDEX i ON t (n);

execute dbms_stats.gather_table_stats(user,'t')

CREATE OR REPLACE TYPE number_t AS TABLE OF number;
/

CREATE OR REPLACE function f(p_count NUMBER) RETURN number_t PIPELINED AS
BEGIN
  FOR i in 1..p_count
  LOOP
    pipe row(i);
  END LOOP;
  RETURN;
END;
/

PAUSE

REM 
REM First execution (without cardinality feedback)
REM 

SELECT * 
FROM t, table(f(10)) f 
WHERE t.n = f.column_value;

SELECT * FROM table(dbms_xplan.display_cursor(format=>'basic rows'));

PAUSE

REM
REM Second execution (with cardinality feedback)
REM

SELECT * 
FROM t, table(f(10)) f 
WHERE t.n = f.column_value;

SELECT * FROM table(dbms_xplan.display_cursor(format=>'basic rows'));

PAUSE

REM
REM find out sql_id
REM

SELECT * 
FROM t, table(f(10)) f 
WHERE t.n = f.column_value;

SELECT prev_sql_id AS sql_id
FROM v$session
WHERE sid = sys_context('userenv','sid');

PAUSE

REM
REM Check the content of the library cache
REM

SELECT child_number, is_shareable
FROM v$sql
WHERE sql_id = '&sql_id';

PAUSE

REM
REM works in 11.2.0.2 only
REM

SELECT child_number, use_feedback_stats
FROM v$sql_shared_cursor
WHERE sql_id = '&sql_id';

PAUSE

REM
REM Cleanup environment
REM

DROP TYPE number_t;
DROP FUNCTION f;
DROP TABLE t PURGE;
