REM ---------------------------------------------------------------------------
REM  Trivadis - http://www.trivadis.com
REM ---------------------------------------------------------------------------
REM  Group/Privileges.: User
REM  Script-Name......: ex_comparing_statistics.sql
REM  Developer........: Christian Antognini (christian.antognini@trivadis.com)
REM  Date.............: November 2007
REM  Version..........: Oracle Database 11g Release 1 and Release 2
REM  Description......: This script shows how to compare different set of 
REM                     statistics with the DBMS_STATS package.
REM  Usage............: 
REM                     
REM  Input parameters.: 
REM  Output...........: 
REM  Called by........:
REM  Requirements.....: The user executing the script must have the privilege
REM                     to execute the DBMS_LOCK package. If the privilege is
REM                     not available, "manually" wait 15 seconds...
REM  Remarks..........: 
REM ---------------------------------------------------------------------------
REM Changes:
REM DD.MM.YYYY Developer Change
REM ---------------------------------------------------------------------------
REM 12.12.2011 ChA       Changed header and added cleanup
REM ---------------------------------------------------------------------------

SET ECHO ON TERMOUT ON SERVEROUTPUT OFF FEEDBACK OFF LONG 1000000 PAGESIZE 100 LINESIZE 80

DROP TABLE t PURGE;

CLEAR SCREEN

REM
REM create test table and gather statistics
REM

CREATE TABLE t
AS
SELECT rownum AS id,
       round(dbms_random.normal*1000) AS val1,
       10 + round(ln(rownum/2+2)) AS val2,
       dbms_random.string('p',25) AS pad
FROM all_objects
WHERE rownum <= 10000
ORDER BY dbms_random.value;

ALTER TABLE t ADD CONSTRAINT t_pk PRIMARY KEY (id);

PAUSE

BEGIN
  dbms_stats.gather_table_stats(
    ownname          => user,
    tabname          => 'T',
    estimate_percent => 100,
    method_opt       => 'for all columns size skewonly',
    cascade          => TRUE
  );
END;
/

PAUSE

REM
REM insert more data
REM

INSERT INTO t
SELECT 10000+id, val1, val2, pad
FROM t
WHERE rownum <= 2500
ORDER BY dbms_random.value;

COMMIT;

PAUSE

REM
REM gather pending object statistics
REM

BEGIN
  dbms_stats.set_table_prefs(user,'t','publish','false');
  dbms_stats.gather_table_stats(
    ownname          => user,
    tabname          => 'T',
    estimate_percent => 100,
    method_opt       => 'for all columns size skewonly',
    cascade          => TRUE
  );
  dbms_stats.set_table_prefs(user,'t','publish','true');
END;
/

PAUSE

REM
REM compare current statistics with pending statistics
REM

SELECT *
FROM table(dbms_stats.diff_table_stats_in_pending(ownname => user,
                                                  tabname => 'T', 
                                                  time_stamp => NULL,
                                                  pctthreshold => 10));

PAUSE

REM
REM wait 15 seconds and then publish pending statistics
REM

execute dbms_lock.sleep(15)

execute dbms_stats.publish_pending_stats(ownname => user, tabname => 'T');

PAUSE

REM
REM compare current statistics with statistics of 15 seconds ago
REM

SELECT *
FROM table(dbms_stats.diff_table_stats_in_history(
             ownname      => user,
             tabname      => 'T',
             time1        => systimestamp - to_dsinterval('0 00:00:15'),
             time2        => NULL,
             pctthreshold => 10));

PAUSE

REM
REM Cleanup
REM

DROP TABLE t PURGE;
