REM ---------------------------------------------------------------------------
REM  Trivadis - http://www.trivadis.com
REM ---------------------------------------------------------------------------
REM  Group/Privileges.: User
REM  Script-Name......: ex_full_outer_join.sql
REM  Developer........: Christian Antognini (christian.antognini@trivadis.com)
REM  Date.............: November 2007
REM  Version..........: Oracle Database 11g Release 1 and Release 2
REM  Description......: The script shows that with 11g the full outer join 
REM                     syntax is no longer processed with a UNION ALL.
REM  Usage............: 
REM                     
REM  Input parameters.: 
REM  Output...........: 
REM  Called by........:
REM  Requirements.....: 
REM  Remarks..........: 
REM ---------------------------------------------------------------------------
REM Changes:
REM DD.MM.YYYY Developer Change
REM ---------------------------------------------------------------------------
REM 05.04.2008 ChA       Added example with hint native_full_outer_join
REM 12.12.2011 ChA       Changed header and added cleanup
REM ---------------------------------------------------------------------------

SET ECHO ON TERMOUT ON SERVEROUTPUT OFF FEEDBACK OFF PAGESIZE 100 LINESIZE 80

DROP TABLE t2 PURGE;
DROP TABLE t1 PURGE;

CLEAR SCREEN

REM
REM create test tables
REM

CREATE TABLE t1 AS 
SELECT rownum AS id 
FROM all_objects
WHERE rownum <= 5;

execute dbms_stats.gather_table_stats(user,'t1')

CREATE TABLE t2 AS 
SELECT rownum+2 AS id
FROM all_objects
WHERE rownum <= 5;

execute dbms_stats.gather_table_stats(user,'t2')

PAUSE

REM
REM without the new optimization a UNION ALL is used
REM to process a full outer join
REM

ALTER SESSION SET "_optimizer_native_full_outer_join" = OFF;

EXPLAIN PLAN FOR
SELECT *
FROM t1 FULL OUTER JOIN t2 ON t1.id = t2.id;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

PAUSE

REM
REM with the new optimization a UNION ALL is no longer used
REM

ALTER SESSION SET "_optimizer_native_full_outer_join" = FORCE;

PAUSE

EXPLAIN PLAN FOR
SELECT *
FROM t1 FULL OUTER JOIN t2 ON t1.id = t2.id;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

PAUSE

REM
REM two hints related to full outer joins:
REM - native_full_outer_join
REM - no_native_full_outer_join
REM

EXPLAIN PLAN FOR
SELECT /*+ no_native_full_outer_join */ *
FROM t1 FULL OUTER JOIN t2 ON t1.id = t2.id;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

PAUSE

ALTER SESSION SET "_optimizer_native_full_outer_join" = OFF;

EXPLAIN PLAN FOR
SELECT /*+ native_full_outer_join */ *
FROM t1 FULL OUTER JOIN t2 ON t1.id = t2.id;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

PAUSE

REM
REM the new optimization works with equi joins only
REM (hash joins support equi joins only!)
REM

EXPLAIN PLAN FOR
SELECT *
FROM t1 FULL OUTER JOIN t2 ON t1.id BETWEEN t2.id AND t2.id+1;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

PAUSE

REM
REM Cleanup
REM

DROP TABLE t1 PURGE;
DROP TABLE t2 PURGE;
