REM ---------------------------------------------------------------------------
REM  Trivadis - http://www.trivadis.com
REM ---------------------------------------------------------------------------
REM  Group/Privileges.: User
REM  Script-Name......: ex_invisible_index.sql
REM  Developer........: Christian Antognini (christian.antognini@trivadis.com)
REM  Date.............: November 2007
REM  Version..........: Oracle Database 11g Release 1 and Release 2
REM  Description......: This script shows how to change the visibility of an
REM                     index, the impact of visibility on the query 
REM                     optimizer and how to use of the parameter 
REM                     optimizer_use_invisible_indexes.
REM  Usage............: 
REM                     
REM  Input parameters.: 
REM  Output...........: 
REM  Called by........:
REM  Requirements.....: 
REM  Remarks..........: 
REM ---------------------------------------------------------------------------
REM Changes:
REM DD.MM.YYYY Developer Change
REM ---------------------------------------------------------------------------
REM 24.05.2011 ChA       Added example with USE_INVISIBLE_INDEXES hint
REM 12.12.2011 ChA       Changed header + added cleanup
REM ---------------------------------------------------------------------------

SET ECHO ON TERMOUT ON SERVEROUTPUT OFF FEEDBACK OFF PAGESIZE 100 LINESIZE 80

CLEAR SCREEN

ALTER SESSION SET optimizer_use_invisible_indexes = FALSE;

DROP TABLE t PURGE;

CLEAR SCREEN

REM
REM create a test table (incl. primary key and object statistics)
REM

CREATE TABLE t AS 
SELECT rownum AS id, rpad('*',50,'*') AS pad 
FROM all_objects
WHERE rownum <= 1000;

ALTER TABLE t ADD CONSTRAINT t_pk PRIMARY KEY (id);

execute dbms_stats.gather_table_stats(user,'t')

PAUSE

REM
REM obviously the primary key index is used when a restriction is 
REM based on the column composing it
REM

EXPLAIN PLAN FOR SELECT * FROM t WHERE id = 42;

SELECT * FROM table(dbms_xplan.display(format => 'basic'));

PAUSE

REM
REM change the visibility of the index
REM

SELECT visibility FROM user_indexes WHERE index_name = 'T_PK';

ALTER INDEX t_pk INVISIBLE;

SELECT visibility FROM user_indexes WHERE index_name = 'T_PK';

PAUSE

REM
REM check that the query optimizer no longer use the invisible index
REM

EXPLAIN PLAN FOR SELECT * FROM t WHERE id = 42;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

PAUSE

REM
REM also when a hint is specified the index cannot be used
REM

EXPLAIN PLAN FOR SELECT /*+ index(t t_pk) */ * FROM t WHERE id = 42;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

PAUSE

REM
REM the only exception is when USE_INVISIBLE_INDEXES is specified
REM

EXPLAIN PLAN FOR SELECT /*+ use_invisible_indexes */ * FROM t WHERE id = 42;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

PAUSE

REM
REM change the visibility of the index and check its utilization
REM

ALTER INDEX t_pk VISIBLE;

SELECT visibility FROM user_indexes WHERE index_name = 'T_PK';

EXPLAIN PLAN FOR SELECT * FROM t WHERE id = 42;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

PAUSE

REM
REM through the parameter optimizer_use_invisible_indexes it is 
REM possible to instruct the query optimizer to use an invisible index
REM

ALTER INDEX t_pk INVISIBLE;

SELECT visibility FROM user_indexes WHERE index_name = 'T_PK';

ALTER SESSION SET optimizer_use_invisible_indexes = TRUE;

EXPLAIN PLAN FOR SELECT * FROM t WHERE id = 42;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

ALTER SESSION SET optimizer_use_invisible_indexes = FALSE;

set autotrace off

PAUSE

REM
REM Cleanup environment
REM

DROP TABLE t PURGE;
