REM ---------------------------------------------------------------------------
REM  Trivadis - http://www.trivadis.com
REM ---------------------------------------------------------------------------
REM  Group/Privileges.: User
REM  Script-Name......: ex_join_elimination.sql
REM  Developer........: Christian Antognini (christian.antognini@trivadis.com)
REM  Date.............: November 2011
REM  Version..........: Oracle Database 11g Release 1 and Release 2
REM  Description......: The script shows an example of the optimization
REM                     technique called "Join Elimination".
REM  Usage............: 
REM                     
REM  Input parameters.: 
REM  Output...........: 
REM  Called by........: 
REM  Requirements.....: 
REM  Remarks..........: 
REM ---------------------------------------------------------------------------
REM Changes:
REM DD.MM.YYYY Developer Change
REM ---------------------------------------------------------------------------
REM 12.12.2011 ChA       Changed header
REM ---------------------------------------------------------------------------

SET ECHO ON TERMOUT ON SERVEROUTPUT OFF FEEDBACK OFF PAGESIZE 100 LINESIZE 80

DROP VIEW v;
DROP TABLE t2 PURGE;
DROP TABLE t1 PURGE;

CLEAR SCREEN

REM
REM Setup environment
REM

CREATE TABLE t1 AS 
SELECT rownum AS id, mod(rownum,1000) AS n1, mod(rownum,1000) AS n2, rpad('*',50,'*') AS pad
FROM all_objects
WHERE rownum <= 1000;

execute dbms_stats.gather_table_stats(user,'t1')

CREATE TABLE t2 AS 
SELECT rownum AS id, mod(rownum,1000) AS n1, mod(rownum,1000) AS n2, rpad('*',50,'*') AS pad
FROM all_objects
WHERE rownum <= 1000;

execute dbms_stats.gather_table_stats(user,'t2')

PAUSE

CREATE VIEW v AS
SELECT t1.id AS t1_id, t1.n1 AS t1_n, t2.id AS t2_id, t2.n1 AS t2_n
FROM t1, t2
WHERE t1.id = t2.id;

PAUSE

REM
REM Join elimination cannot be used without constraints
REM

EXPLAIN PLAN FOR 
SELECT t2_id, t2_n 
FROM v;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

PAUSE

REM
REM With constraints in place (PK-FK relation) join elination is possible
REM

ALTER TABLE t1 ADD CONSTRAINT t1_pk PRIMARY KEY (id);
ALTER TABLE t2 ADD CONSTRAINT t2_t1_fk FOREIGN KEY (id) REFERENCES t1;

PAUSE

EXPLAIN PLAN FOR 
SELECT t2_id, t2_n 
FROM v;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

PAUSE

REM
REM Cleanup environment
REM

DROP VIEW v;
DROP TABLE t2 PURGE;
DROP TABLE t1 PURGE;
