REM ---------------------------------------------------------------------------
REM  Trivadis - http://www.trivadis.com
REM ---------------------------------------------------------------------------
REM  Group/Privileges.: User
REM  Script-Name......: ex_join_factorization.sql
REM  Developer........: Christian Antognini (christian.antognini@trivadis.com)
REM  Date.............: November 2011
REM  Version..........: Oracle Database 11g Release 2
REM  Description......: The script shows an example of the optimization
REM                     technique called "Join Factorization".
REM  Usage............: 
REM                     
REM  Input parameters.: 
REM  Output...........: 
REM  Called by........:
REM  Requirements.....: 
REM  Remarks..........: 
REM ---------------------------------------------------------------------------
REM Changes:
REM DD.MM.YYYY Developer Change
REM ---------------------------------------------------------------------------
REM 12.12.2011 ChA       Changed header
REM ---------------------------------------------------------------------------

SET ECHO ON TERMOUT ON SERVEROUTPUT OFF FEEDBACK OFF PAGESIZE 100 LINESIZE 80

DROP TABLE t2 PURGE;
DROP TABLE t1 PURGE;

CLEAR SCREEN

REM
REM Setup environment
REM

CREATE TABLE t1 AS 
SELECT rownum AS id, mod(rownum,1000) AS n1, mod(rownum,1000) AS n2, rpad('*',50,'*') AS pad
FROM all_objects
WHERE rownum <= 1000;

execute dbms_stats.gather_table_stats(user,'t1')

CREATE TABLE t2 AS 
SELECT rownum AS id, mod(rownum,1000) AS n1, mod(rownum,1000) AS n2, rpad('*',50,'*') AS pad
FROM all_objects
WHERE rownum <= 1000;

execute dbms_stats.gather_table_stats(user,'t2')

PAUSE

REM
REM Join factorization disabled
REM

EXPLAIN PLAN FOR
SELECT /*+ no_factorize_join(@set$1) */ * 
FROM t1, t2 
WHERE t1.id = t2.id AND t2.id < 10
UNION ALL
SELECT * 
FROM t1, t2 
WHERE t1.id = t2.id AND t2.id > 990;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

PAUSE

REM
REM Join factorization enabled
REM

EXPLAIN PLAN FOR
SELECT /*+ factorize_join(@set$1) */ * 
FROM t1, t2 
WHERE t1.id = t2.id AND t2.id < 10
UNION ALL
SELECT * 
FROM t1, t2 
WHERE t1.id = t2.id AND t2.id > 990;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

PAUSE

REM
REM Cleanup environment
REM

DROP TABLE t2 PURGE;
DROP TABLE t1 PURGE;
