REM ---------------------------------------------------------------------------
REM  Trivadis - http://www.trivadis.com
REM ---------------------------------------------------------------------------
REM  Group/Privileges.: User
REM  Script-Name......: ex_join_filter_pruning.sql
REM  Developer........: Christian Antognini (christian.antognini@trivadis.com)
REM  Date.............: September 2007
REM  Version..........: Oracle Database 11g Release 1 and Release 2
REM  Description......: The script shows an example of subquery pruning and of
REM                     join filter pruning.
REM  Usage............: 
REM                     
REM  Input parameters.: 
REM  Output...........: 
REM  Called by........:
REM  Requirements.....: 
REM  Remarks..........: 
REM ---------------------------------------------------------------------------
REM Changes:
REM DD.MM.YYYY Developer Change
REM ---------------------------------------------------------------------------
REM 12.12.2011 ChA       Changed header + added cleanup
REM ---------------------------------------------------------------------------

SET ECHO ON TERMOUT ON SERVEROUTPUT OFF FEEDBACK OFF LONG 1000 PAGESIZE 100 LINESIZE 80

DROP TABLE t PURGE;

COLUMN id FORMAT 99
COLUMN other FORMAT A70

CLEAR SCREEN

REM
REM create test table, insert data and gather statistics
REM

CREATE TABLE t (
  id NUMBER,
  d DATE,
  pad VARCHAR2(4000),
  CONSTRAINT t_pk PRIMARY KEY (id)
)
PARTITION BY RANGE (d) (
  PARTITION t_jan_2007 VALUES LESS THAN (to_date('2007-02-01','yyyy-mm-dd')),
  PARTITION t_feb_2007 VALUES LESS THAN (to_date('2007-03-01','yyyy-mm-dd')),
  PARTITION t_mar_2007 VALUES LESS THAN (to_date('2007-04-01','yyyy-mm-dd')),
  PARTITION t_apr_2007 VALUES LESS THAN (to_date('2007-05-01','yyyy-mm-dd')),
  PARTITION t_may_2007 VALUES LESS THAN (to_date('2007-06-01','yyyy-mm-dd')),
  PARTITION t_jun_2007 VALUES LESS THAN (to_date('2007-07-01','yyyy-mm-dd')),
  PARTITION t_jul_2007 VALUES LESS THAN (to_date('2007-08-01','yyyy-mm-dd')),
  PARTITION t_aug_2007 VALUES LESS THAN (to_date('2007-09-01','yyyy-mm-dd')),
  PARTITION t_sep_2007 VALUES LESS THAN (to_date('2007-10-01','yyyy-mm-dd')),
  PARTITION t_oct_2007 VALUES LESS THAN (to_date('2007-11-01','yyyy-mm-dd')),
  PARTITION t_nov_2007 VALUES LESS THAN (to_date('2007-12-01','yyyy-mm-dd')),
  PARTITION t_dec_2007 VALUES LESS THAN (to_date('2008-01-01','yyyy-mm-dd'))
);

PAUSE

INSERT INTO t
SELECT rownum AS id,
       trunc(to_date('2007-01-01','yyyy-mm-dd')+rownum/27.4) AS d,
       rpad('*',50,'*') AS pad
FROM all_objects
WHERE rownum <= 10000
ORDER BY dbms_random.value;

PAUSE

execute dbms_stats.gather_table_stats(user, 't')

PAUSE

REM
REM with a nested loop join a "regular" pruning is performed
REM

EXPLAIN PLAN FOR
SELECT /*+ ordered use_nl(t2) */ * 
FROM t t1, t t2 
WHERE t1.d = t2.d AND t1.id = 19;

PAUSE

SELECT * FROM table(dbms_xplan.display(NULL,NULL,'basic partition'));

PAUSE

REM
REM up to in 10gR2 with a hash join subquery pruning is performed,
REM therefore a recursive query is executed 
REM

ALTER SESSION SET "_bloom_pruning_enabled" = FALSE;
ALTER SESSION SET "_subquery_pruning_enabled" = TRUE;
ALTER SESSION SET "_subquery_pruning_cost_factor"=1;
ALTER SESSION SET "_subquery_pruning_reduction"=100;

EXPLAIN PLAN FOR
SELECT /*+ ordered use_hash(t2) */ * 
FROM t t1, t t2 
WHERE t1.d = t2.d AND t1.id = 19;

PAUSE

SELECT * FROM table(dbms_xplan.display(NULL,NULL,'basic partition'));

PAUSE

SELECT id, other 
FROM plan_table 
WHERE other IS NOT NULL 
AND plan_id = (SELECT max(plan_id) FROM plan_table);

PAUSE

REM
REM as of 11g join to avoid the recursive query join filter pruning
REM is available as well
REM

ALTER SESSION SET "_bloom_pruning_enabled" = TRUE;

EXPLAIN PLAN FOR
SELECT /*+ ordered use_hash(t2) */ * 
FROM t t1, t t2 
WHERE t1.d = t2.d AND t1.id = 19;

PAUSE

SELECT * FROM table(dbms_xplan.display(NULL,NULL,'basic partition'));

PAUSE

REM
REM Cleanup environment
REM

DROP TABLE t PURGE;
