REM ---------------------------------------------------------------------------
REM  Trivadis - http://www.trivadis.com
REM ---------------------------------------------------------------------------
REM  Group/Privileges.: User
REM  Script-Name......: ex_linguistic_like.sql
REM  Developer........: Christian Antognini (christian.atnognini@trivadis.com)
REM  Date.............: November 2007
REM  Version..........: Oracle Database 11g Release 1 and Release 2
REM  Description......: This script shows that a linguistic function-based
REM                     index can be used with the LIKE operator.
REM  Usage............: 
REM                     
REM  Input parameters.: 
REM  Output...........: 
REM  Called by........:
REM  Requirements.....: 
REM  Remarks..........: Run the script on both 10g and 11g to see the difference.
REM ---------------------------------------------------------------------------
REM Changes:
REM DD.MM.YYYY Developer Change
REM ---------------------------------------------------------------------------
REM 12.12.2011 ChA       Changed header + added cleanup
REM ---------------------------------------------------------------------------

SET ECHO ON TERMOUT ON SERVEROUTPUT OFF FEEDBACK OFF PAGESIZE 100 LINESIZE 80

CLEAR SCREEN

DROP TABLE t PURGE;

CLEAR SCREEN

REM
REM display database version
REM

SELECT *
FROM v$version
WHERE rownum = 1;

PAUSE

REM
REM create a test table (incl. object statistics)
REM

CREATE TABLE t AS 
SELECT rownum AS id, rpad('*',25,'*') AS pad 
FROM all_objects
WHERE rownum <= 10;

execute dbms_stats.gather_table_stats(user,'t')

PAUSE

REM
REM create a linguistic (case insensitive) function-based index
REM

CREATE INDEX i ON t(nlssort(pad,'nls_sort=binary_ci'));

PAUSE

REM
REM the index cannot be used if the parameter NLS_COMP is set to BINARY
REM

ALTER SESSION SET nls_sort=binary_ci;
ALTER SESSION SET nls_comp=binary;

EXPLAIN PLAN FOR SELECT * FROM t WHERE pad = 'A';

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

PAUSE

REM
REM the index can be used if the parameter NLS_COMP is set to LINGUISTIC
REM

ALTER SESSION SET nls_comp=linguistic;

EXPLAIN PLAN FOR SELECT * FROM t WHERE pad = 'A';

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

PAUSE

REM
REM only as of 11g a LIKE can use the index
REM

EXPLAIN PLAN FOR SELECT * FROM t WHERE pad LIKE 'A%';

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

PAUSE

REM
REM Cleanup environment
REM

DROP TABLE t PURGE;
