REM ---------------------------------------------------------------------------
REM  Trivadis - http://www.trivadis.com
REM ---------------------------------------------------------------------------
REM  Group/Privileges.: User
REM  Script-Name......: ex_or_expansion.sql
REM  Developer........: Christian Antognini (christian.antognini@trivadis.com)
REM  Date.............: November 2011
REM  Version..........: Oracle Database 11g Release 2
REM  Description......: The script shows an example of the optimization
REM                     technique called "OR Expansion". This optimization is 
REM                     available for function-based indexes as of 11.2.0.2
REM                     only.
REM  Usage............: 
REM                     
REM  Input parameters.: 
REM  Output...........: 
REM  Called by........:
REM  Requirements.....: 
REM  Remarks..........: 
REM ---------------------------------------------------------------------------
REM Changes:
REM DD.MM.YYYY Developer Change
REM ---------------------------------------------------------------------------
REM 12.12.2011 ChA       Changed header
REM ---------------------------------------------------------------------------

SET ECHO ON TERMOUT ON SERVEROUTPUT OFF FEEDBACK OFF PAGESIZE 100 LINESIZE 80

DROP TABLE t PURGE;

CLEAR SCREEN

REM
REM Setup environment
REM

CREATE TABLE t AS 
SELECT rownum AS id, mod(rownum,1000) AS n1, mod(rownum,1000) AS n2, rpad('*',50,'*') AS pad
FROM all_objects
WHERE rownum <= 1000;

CREATE INDEX i1_floor ON t (floor(n1));
CREATE INDEX i2_floor ON t (floor(n2));

execute dbms_stats.gather_table_stats(user,'t')

ALTER SESSION SET "_b_tree_bitmap_plans" = FALSE;

PAUSE

REM
REM OR expansion disabled
REM

EXPLAIN PLAN FOR
SELECT /*+ no_expand */ *
FROM t
WHERE floor(n1) = 8 OR floor(n2) = 4;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

PAUSE

REM
REM OR expansion enabled (with an FBI it works as of 11.2.0.2 only)
REM

EXPLAIN PLAN FOR
SELECT /*+ or_expand */ *
FROM t
WHERE floor(n1) = 8 OR floor(n2) = 4;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

PAUSE

REM
REM Cleanup environment
REM

DROP TABLE t PURGE;
