REM ---------------------------------------------------------------------------
REM  Trivadis - http://www.trivadis.com
REM ---------------------------------------------------------------------------
REM  Group/Privileges.: User
REM  Script-Name......: ex_partition_pruning_ctas.sql
REM  Developer........: MaM (maurice.mueller@trivadis.com)
REM  Date.............: October 2007
REM  Version..........: Oracle Database 11g Release 1 and Release 2
REM  Description......: Create several tables used by the script 
REM                     ex_partition_pruning.sql to show different type of
REM                     partition pruning.
REM  Usage............: 
REM                     
REM  Input parameters.: 
REM  Output...........: 
REM  Called by........:
REM  Requirements.....: Sample schema SH
REM  Remarks..........: 
REM ---------------------------------------------------------------------------
REM Changes:
REM DD.MM.YYYY Developer Change
REM ---------------------------------------------------------------------------
REM 13.11.2007 ChA       Formatting + reduction of the number of tables
REM 12.12.2011 ChA       Changed header
REM ---------------------------------------------------------------------------

SET ECHO ON TERMOUT ON SERVEROUTPUT ON PAGESIZE 100 LINESIZE 80

CLEAR SCREEN

REM
REM dimensions
REM

DROP TABLE d_times PURGE;
CREATE TABLE d_times AS SELECT * FROM sh.times;

DROP TABLE d_countries PURGE;
CREATE TABLE d_countries AS SELECT * FROM sh.countries;

DROP TABLE d_customers PURGE;
CREATE TABLE d_customers COMPRESS AS SELECT * FROM sh.customers;

REM
REM range partitioned
REM

DROP TABLE f_sales_r PURGE;

CREATE TABLE f_sales_r COMPRESS
PARTITION BY RANGE (time_id) 
(
  PARTITION sales_1998 VALUES LESS THAN (to_date('19990101', 'YYYYMMDD')) COMPRESS,      
  PARTITION sales_1999 VALUES LESS THAN (to_date('20000101', 'YYYYMMDD')) COMPRESS,
  PARTITION sales_2000 VALUES LESS THAN (to_date('20010101', 'YYYYMMDD')) COMPRESS
)
AS SELECT s.cust_id, s.time_id, s.amount_sold
FROM sh.sales s 
WHERE time_id < to_date('20010101', 'YYYYMMDD');

REM
REM range partitioned, hash subpartitioned
REM

DROP TABLE f_sales_rh PURGE;

CREATE TABLE f_sales_rh	COMPRESS
PARTITION BY RANGE (time_id) 
SUBPARTITION BY HASH (country_id) SUBPARTITIONS 4
(
  PARTITION sales_1998 VALUES LESS THAN (to_date('19990101', 'YYYYMMDD')),      
  PARTITION sales_1999 VALUES LESS THAN (to_date('20000101', 'YYYYMMDD')),
  PARTITION sales_2000 VALUES LESS THAN (to_date('20010101', 'YYYYMMDD')) 
)
AS SELECT c.country_id, s.cust_id, s.time_id, s.amount_sold
FROM sh.sales s, sh.customers c
WHERE s.cust_id = c.cust_id
AND time_id < to_date('20010101', 'YYYYMMDD');

REM
REM range partitioned, list subpartitioned
REM

DROP TABLE f_sales_rl PURGE;

CREATE TABLE f_sales_rl	COMPRESS
PARTITION BY RANGE (time_id) 
SUBPARTITION BY LIST (country_id) 
SUBPARTITION TEMPLATE 
(
  SUBPARTITION us    VALUES (52790),
  SUBPARTITION de    VALUES (52776),
  SUBPARTITION fr    VALUES (52779),   
  SUBPARTITION other VALUES (DEFAULT)
)
(
  PARTITION sales_1998 VALUES LESS THAN (to_date('19990101', 'YYYYMMDD')) ,      
  PARTITION sales_1999 VALUES LESS THAN (to_date('20000101', 'YYYYMMDD')) ,
  PARTITION sales_2000 VALUES LESS THAN (to_date('20010101', 'YYYYMMDD')) 
)
AS SELECT c.country_id, s.cust_id, s.time_id, s.amount_sold
FROM sh.sales s, sh.customers c
WHERE s.cust_id = c.cust_id
AND time_id < to_date('20010101', 'YYYYMMDD');

REM
REM range partitioned, range subpartitioned
REM

DROP TABLE f_sales_rr PURGE;

CREATE TABLE f_sales_rr COMPRESS
PARTITION BY RANGE (order_time_id)  
SUBPARTITION BY RANGE (pay_time_id)
(
  PARTITION sales_1999 VALUES LESS THAN (to_date('20000101', 'YYYYMMDD'))   
  (
    SUBPARTITION sales_1999_1999     VALUES LESS THAN (to_date('20000101', 'YYYYMMDD')), 
    SUBPARTITION sales_1999_maxvalue VALUES LESS THAN (MAXVALUE)
  ),
  PARTITION sales_2000 VALUES LESS THAN (to_date('20010101', 'YYYYMMDD')) 
  ( 
    SUBPARTITION sales_2000_2000     VALUES LESS THAN (to_date('20010101', 'YYYYMMDD')),
    SUBPARTITION sales_2000_maxvalue VALUES LESS THAN (MAXVALUE)
  ),
  PARTITION sales_maxvalue VALUES LESS THAN (MAXVALUE) 
  (
    SUBPARTITION sales_maxvalue_maxvalue VALUES LESS THAN (MAXVALUE)
  )
)
AS SELECT s.cust_id, s.time_id order_time_id, s.time_id+mod(rownum,100)*3 pay_time_id, s.amount_sold
FROM sh.sales s 
WHERE s.time_id < to_date('20020101', 'YYYYMMDD');

REM
REM hash partitioned
REM

DROP TABLE f_sales_h PURGE;

CREATE TABLE f_sales_h COMPRESS
PARTITION BY HASH (country_id) PARTITIONS 4
AS SELECT c.country_id, s.cust_id, s.time_id, s.amount_sold
FROM sh.sales s, sh.customers c
WHERE s.cust_id = c.cust_id
AND time_id < to_date('20010101', 'YYYYMMDD');

REM
REM list partitioned, range subpartitioned
REM

DROP TABLE f_sales_lr PURGE;

CREATE TABLE f_sales_lr	COMPRESS
PARTITION BY LIST (country_id) 
SUBPARTITION BY RANGE (time_id)
SUBPARTITION TEMPLATE 
(
  SUBPARTITION sales_1998 VALUES LESS THAN (to_date('19990101', 'YYYYMMDD')),
  SUBPARTITION sales_1999 VALUES LESS THAN (to_date('20000101', 'YYYYMMDD')),
  SUBPARTITION sales_2000 VALUES LESS THAN (to_date('20010101', 'YYYYMMDD'))
)
(
  PARTITION us    VALUES (52790),
  PARTITION de    VALUES (52776),
  PARTITION fr    VALUES (52779),   
  PARTITION other VALUES (DEFAULT)
)
AS SELECT c.country_id, s.cust_id, s.time_id, s.amount_sold
FROM sh.sales s, sh.customers c
WHERE s.cust_id = c.cust_id
AND time_id < to_date('20010101', 'YYYYMMDD');

REM
REM list partitioned, hash subpartitioned
REM

DROP TABLE f_sales_lh PURGE;

CREATE TABLE f_sales_lh	COMPRESS
PARTITION BY LIST (country_id) 
SUBPARTITION BY HASH (cust_id) SUBPARTITIONS 4
(
  PARTITION us    VALUES (52790),
  PARTITION de    VALUES (52776),
  PARTITION fr    VALUES (52779),   
  PARTITION other VALUES (DEFAULT)
)
AS SELECT c.country_id, s.cust_id, s.time_id, s.amount_sold
FROM sh.sales s, sh.customers c
WHERE s.cust_id = c.cust_id
AND time_id < to_date('20010101', 'YYYYMMDD');

REM
REM list partitioned, list subpartitioned
REM

DROP TABLE f_sales_ll PURGE;

CREATE TABLE f_sales_ll	COMPRESS
PARTITION BY LIST (in_country_id) 
SUBPARTITION BY LIST (out_country_id)
SUBPARTITION TEMPLATE 
(
  SUBPARTITION us    VALUES (52790),
  SUBPARTITION de    VALUES (52776),
  SUBPARTITION fr    VALUES (52779),   
  SUBPARTITION other VALUES (DEFAULT)
)
(
  PARTITION us    VALUES (52790),
  PARTITION de    VALUES (52776),
  PARTITION fr    VALUES (52779),   
  PARTITION other VALUES (DEFAULT)
)
AS SELECT c.country_id in_country_id, c.country_id+ mod(rownum,15) out_country_id, s.time_id, s.amount_sold
FROM sh.sales s, sh.customers c
WHERE s.cust_id = c.cust_id
AND time_id < to_date('20010101', 'YYYYMMDD');


execute dbms_stats.gather_schema_stats(ownname => user, options => 'GATHER AUTO')

SET ECHO OFF
