REM ---------------------------------------------------------------------------
REM  Trivadis - http://www.trivadis.com
REM ---------------------------------------------------------------------------
REM  Group/Privileges.: User
REM  Script-Name......: ex_pending_object_statistics.sql
REM  Developer........: Christian Antognini (christian.antognini@trivadis.com)
REM  Date.............: November 2007
REM  Version..........: Oracle Database 11g Release 1 and Release 2
REM  Description......: The script shows how to gather, use and publish pending
REM                     statistics.
REM  Usage............: 
REM                     
REM  Input parameters.: 
REM  Output...........: 
REM  Called by........:
REM  Requirements.....: 
REM  Remarks..........: 
REM ---------------------------------------------------------------------------
REM Changes:
REM DD.MM.YYYY Developer Change
REM ---------------------------------------------------------------------------
REM 12.12.2011 ChA       Changed header + added cleanup
REM ---------------------------------------------------------------------------

SET ECHO ON TERMOUT ON SERVEROUTPUT OFF FEEDBACK OFF PAGESIZE 100 LINESIZE 80

DROP TABLE t PURGE;

CLEAR SCREEN

REM
REM create test table, gather object statistics and check their utilization
REM

execute dbms_random.seed(0)

CREATE TABLE t
AS
SELECT rownum AS id,
       dbms_random.string('p',250) AS pad
FROM all_objects
WHERE rownum <= 1000
ORDER BY dbms_random.value;

PAUSE

BEGIN
  dbms_stats.set_table_prefs(user,'t','publish','true');
  dbms_stats.gather_table_stats(
    ownname          => user,
    tabname          => 'T',
    estimate_percent => 100,
    method_opt       => 'for all columns size 1',
    cascade          => TRUE
  );
END;
/

SELECT num_rows FROM user_tables WHERE table_name = 'T';

SELECT num_rows FROM user_tab_pending_stats WHERE table_name = 'T';

PAUSE

EXPLAIN PLAN FOR SELECT * FROM t;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic rows'));

PAUSE

REM
REM insert more data into the test table and gather pending statistics
REM

INSERT INTO t SELECT 1000+id, pad FROM t;

COMMIT;

PAUSE

BEGIN
  dbms_stats.set_table_prefs(user,'t','publish','false');
  dbms_stats.gather_table_stats(
    ownname          => user,
    tabname          => 'T',
    estimate_percent => 100,
    method_opt       => 'for all columns size 1',
    cascade          => TRUE
  );
  dbms_stats.set_table_prefs(user,'t','publish','true');
END;
/

SELECT num_rows FROM user_tables WHERE table_name = 'T';

SELECT num_rows FROM user_tab_pending_stats WHERE table_name = 'T';

PAUSE

REM
REM use current object statistics
REM

ALTER SESSION SET optimizer_use_pending_statistics = FALSE;

EXPLAIN PLAN FOR SELECT * FROM t;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic rows'));

PAUSE

REM
REM use pending object statistics
REM

ALTER SESSION SET optimizer_use_pending_statistics = TRUE;

EXPLAIN PLAN FOR SELECT * FROM t;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic rows'));

PAUSE 

REM
REM publish pending statistics and check their utilization
REM 

SELECT num_rows FROM user_tables WHERE table_name = 'T';

execute dbms_stats.publish_pending_stats(ownname => user, tabname => 'T');

SELECT num_rows FROM user_tables WHERE table_name = 'T';

ALTER SESSION SET optimizer_use_pending_statistics = FALSE;

PAUSE

EXPLAIN PLAN FOR SELECT * FROM t;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic rows'));

PAUSE

REM
REM Cleanup
REM

DROP TABLE t PURGE;
