REM ---------------------------------------------------------------------------
REM  Trivadis - http://www.trivadis.com
REM ---------------------------------------------------------------------------
REM  Group/Privileges.: User
REM  Script-Name......: ex_subquery_unnesting.sql
REM  Developer........: Christian Antognini (christian.antognini@trivadis.com)
REM  Date.............: November 2011
REM  Version..........: Oracle Database 11g Release 1 and Release 2
REM  Description......: The script provides several examples of subquery
REM                     unnesting with set operators.
REM  Usage............: 
REM                     
REM  Input parameters.: 
REM  Output...........: 
REM  Called by........: 
REM  Requirements.....: 
REM  Remarks..........: 
REM ---------------------------------------------------------------------------
REM Changes:
REM DD.MM.YYYY Developer Change
REM ---------------------------------------------------------------------------
REM 12.12.2011 ChA       Changed header
REM ---------------------------------------------------------------------------

SET TERMOUT ON
SET FEEDBACK OFF
SET VERIFY OFF
SET SCAN ON
SET ECHO ON
SET LINESIZE 100
SET PAGESIZE 100

DROP TABLE t2 PURGE;
DROP TABLE t1 PURGE;

CLEAR SCREEN

REM
REM Setup test environment
REM

CREATE TABLE t1 AS 
SELECT rownum AS id, mod(rownum,1000) AS n1, mod(rownum,1000) AS n2, rpad('*',50,'*') AS pad
FROM all_objects
WHERE rownum <= 1000;

execute dbms_stats.gather_table_stats(user,'t1')

CREATE TABLE t2 AS 
SELECT rownum AS id, rownum AS t1_id, mod(rownum,1000) AS n1, mod(rownum,1000) AS n2, rpad('*',50,'*') AS pad
FROM all_objects
WHERE rownum <= 1000;

execute dbms_stats.gather_table_stats(user,'t2')

PAUSE

REM
REM UNION ALL
REM

EXPLAIN PLAN FOR
SELECT *
FROM t2
WHERE EXISTS (SELECT /*+ no_unnest */ 1
              FROM t1
              WHERE t1.id = t2.t1_id
              AND t1.pad IS NOT NULL
              UNION ALL
              SELECT 1
              FROM t1
              WHERE t1.id = t2.t1_id
              AND t1.pad IS NULL);

SELECT * FROM table(dbms_xplan.display);

PAUSE

EXPLAIN PLAN FOR
SELECT *
FROM t2
WHERE EXISTS (SELECT /*+ unnest */ 1
              FROM t1
              WHERE t1.id = t2.t1_id
              AND t1.pad IS NOT NULL
              UNION ALL
              SELECT 1
              FROM t1
              WHERE t1.id = t2.t1_id
              AND t1.pad IS NULL);

SELECT * FROM table(dbms_xplan.display);

PAUSE

REM
REM UNION
REM

EXPLAIN PLAN FOR
SELECT *
FROM t2
WHERE EXISTS (SELECT /*+ no_unnest */ 1
              FROM t1
              WHERE t1.id = t2.t1_id
              AND t1.pad IS NOT NULL
              UNION
              SELECT 1
              FROM t1
              WHERE t1.id = t2.t1_id
              AND t1.pad IS NULL);

SELECT * FROM table(dbms_xplan.display);

PAUSE

EXPLAIN PLAN FOR
SELECT *
FROM t2
WHERE EXISTS (SELECT /*+ unnest */ 1
              FROM t1
              WHERE t1.id = t2.t1_id
              AND t1.pad IS NOT NULL
              UNION
              SELECT 1
              FROM t1
              WHERE t1.id = t2.t1_id
              AND t1.pad IS NULL);

SELECT * FROM table(dbms_xplan.display);

PAUSE

REM
REM MINUS
REM

EXPLAIN PLAN FOR
SELECT *
FROM t2
WHERE EXISTS (SELECT /*+ no_unnest */ 1
              FROM t1
              WHERE t1.id = t2.t1_id
              AND t1.pad IS NOT NULL
              MINUS
              SELECT 1
              FROM t1
              WHERE t1.id = t2.t1_id
              AND t1.pad IS NULL);

SELECT * FROM table(dbms_xplan.display);

PAUSE

EXPLAIN PLAN FOR
SELECT *
FROM t2
WHERE EXISTS (SELECT /*+ unnest */ 1
              FROM t1
              WHERE t1.id = t2.t1_id
              AND t1.pad IS NOT NULL
              MINUS
              SELECT 1
              FROM t1
              WHERE t1.id = t2.t1_id
              AND t1.pad IS NULL);

SELECT * FROM table(dbms_xplan.display);

PAUSE

REM
REM INTERSECT
REM

EXPLAIN PLAN FOR
SELECT *
FROM t2
WHERE EXISTS (SELECT /*+ no_unnest */ 1
              FROM t1
              WHERE t1.id = t2.t1_id
              AND t1.pad IS NOT NULL
              INTERSECT
              SELECT 1
              FROM t1
              WHERE t1.id = t2.t1_id
              AND t1.pad IS NULL);

SELECT * FROM table(dbms_xplan.display);

PAUSE

EXPLAIN PLAN FOR
SELECT *
FROM t2
WHERE EXISTS (SELECT /*+ unnest */ 1
              FROM t1
              WHERE t1.id = t2.t1_id
              AND t1.pad IS NOT NULL
              INTERSECT
              SELECT 1
              FROM t1
              WHERE t1.id = t2.t1_id
              AND t1.pad IS NULL);

SELECT * FROM table(dbms_xplan.display);

PAUSE

REM
REM Cleanup
REM

DROP TABLE t2 PURGE;
DROP TABLE t1 PURGE;
