REM ---------------------------------------------------------------------------
REM  Trivadis - http://www.trivadis.com
REM ---------------------------------------------------------------------------
REM  Group/Privileges.: User
REM  Script-Name......: ex_table_expansion.sql
REM  Developer........: Christian Antognini (christian.antognini@trivadis.com)
REM  Date.............: November 2011
REM  Version..........: Oracle Database 11g Release 2
REM  Description......: The script shows an example of the optimization
REM                     technique called "Table Expansion".
REM  Usage............: 
REM                     
REM  Input parameters.: 
REM  Output...........: 
REM  Called by........:
REM  Requirements.....: 
REM  Remarks..........: 
REM ---------------------------------------------------------------------------
REM Changes:
REM DD.MM.YYYY Developer Change
REM ---------------------------------------------------------------------------
REM 12.12.2011 ChA       Changed header
REM ---------------------------------------------------------------------------

SET ECHO ON TERMOUT ON SERVEROUTPUT OFF FEEDBACK OFF PAGESIZE 100 LINESIZE 80

DROP TABLE t PURGE;

CLEAR SCREEN

REM
REM Setup environment
REM

CREATE TABLE t (
  id NUMBER NOT NULL,
  d DATE NOT NULL,
  n NUMBER NOT NULL,
  pad VARCHAR2(4000) NOT NULL
)
PARTITION BY RANGE (d) (
  PARTITION t_jan_2009 VALUES LESS THAN (to_date('2009-02-01','yyyy-mm-dd')),
  PARTITION t_feb_2009 VALUES LESS THAN (to_date('2009-03-01','yyyy-mm-dd')),
  PARTITION t_mar_2009 VALUES LESS THAN (to_date('2009-04-01','yyyy-mm-dd')),
  PARTITION t_apr_2009 VALUES LESS THAN (to_date('2009-05-01','yyyy-mm-dd')),
  PARTITION t_may_2009 VALUES LESS THAN (to_date('2009-06-01','yyyy-mm-dd')),
  PARTITION t_jun_2009 VALUES LESS THAN (to_date('2009-07-01','yyyy-mm-dd')),
  PARTITION t_jul_2009 VALUES LESS THAN (to_date('2009-08-01','yyyy-mm-dd')),
  PARTITION t_aug_2009 VALUES LESS THAN (to_date('2009-09-01','yyyy-mm-dd')),
  PARTITION t_sep_2009 VALUES LESS THAN (to_date('2009-10-01','yyyy-mm-dd')),
  PARTITION t_oct_2009 VALUES LESS THAN (to_date('2009-11-01','yyyy-mm-dd')),
  PARTITION t_nov_2009 VALUES LESS THAN (to_date('2009-12-01','yyyy-mm-dd')),
  PARTITION t_dec_2009 VALUES LESS THAN (to_date('2010-01-01','yyyy-mm-dd'))
);

INSERT INTO t
SELECT rownum, to_date('2009-01-01','yyyy-mm-dd')+rownum/274, mod(rownum,11), rpad('*',100,'*')
FROM dual
CONNECT BY level <= 100000;

CREATE INDEX i ON t (d) LOCAL;

execute dbms_stats.gather_table_stats(user,'t')

PAUSE

REM
REM Status of index partitions
REM

ALTER INDEX i MODIFY PARTITION t_jan_2009 UNUSABLE;
ALTER INDEX i MODIFY PARTITION t_feb_2009 UNUSABLE;
ALTER INDEX i MODIFY PARTITION t_mar_2009 UNUSABLE;
ALTER INDEX i MODIFY PARTITION t_apr_2009 UNUSABLE;

PAUSE

SELECT partition_name, status
FROM user_ind_partitions 
WHERE index_name = 'I' 
ORDER BY partition_position;

PAUSE

REM
REM Table expansion disabled
REM

EXPLAIN PLAN FOR
SELECT /*+ no_expand_table(t) */ count(d)
FROM t
WHERE d BETWEEN to_date('2009-04-30 23:00','yyyy-mm-dd hh24:mi')
            AND to_date('2009-05-02 01:00','yyyy-mm-dd hh24:mi');

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic +partition'));

PAUSE

REM
REM Table expansion enable
REM

EXPLAIN PLAN FOR
SELECT /*+ expand_table(t) */ count(d)
FROM t
WHERE d BETWEEN to_date('2009-04-30 23:00','yyyy-mm-dd hh24:mi')
            AND to_date('2009-05-02 01:00','yyyy-mm-dd hh24:mi');

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic +partition'));

PAUSE

REM
REM Cleanup environment
REM

DROP TABLE t PURGE;
