REM ---------------------------------------------------------------------------
REM  Trivadis - http://www.trivadis.com
REM ---------------------------------------------------------------------------
REM  Group/Privileges.: User
REM  Script-Name......: ex_virtual_column_pruning.sql
REM  Developer........: Christian Antognini (christian.antognini@trivadis.com)
REM  Date.............: November 2007
REM  Version..........: Oracle Database 11g Release 1 and Release 2
REM  Description......: This script shows that partition pruning works for both
REM                     the virtual column and the function defining it.
REM  Usage............: 
REM                     
REM  Input parameters.: 
REM  Output...........: 
REM  Called by........:
REM  Requirements.....: 
REM  Remarks..........: 
REM ---------------------------------------------------------------------------
REM Changes:
REM DD.MM.YYYY Developer Change
REM ---------------------------------------------------------------------------
REM 12.12.2011 ChA       Changed header + added cleanup
REM ---------------------------------------------------------------------------

SET ECHO ON TERMOUT ON SERVEROUTPUT OFF FEEDBACK OFF PAGESIZE 100 LINESIZE 80

DROP TABLE t PURGE;

CLEAR SCREEN

REM
REM create test table, insert data and gather statistics
REM

CREATE TABLE t (
  id NUMBER,
  pkey AS (mod(id,4)),
  pad VARCHAR2(100)
)
PARTITION BY LIST (pkey)
(
  PARTITION t_0 VALUES (0),
  PARTITION t_1 VALUES (1),
  PARTITION t_2 VALUES (2),
  PARTITION t_3 VALUES (3)
);

PAUSE

INSERT INTO t (id, pad) 
SELECT rownum, rpad('*',50,'*')
FROM all_objects
WHERE rownum <= 1000;

execute dbms_stats.gather_table_stats(user,'t')

PAUSE

REM
REM partition pruning with a restriction based on the virtual column
REM

EXPLAIN PLAN FOR SELECT * FROM t WHERE pkey = 1;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic partition'));

PAUSE

REM
REM partition pruning with a restriction based on the function
REM defining the virtual column
REM

EXPLAIN PLAN FOR SELECT * FROM t WHERE mod(id,4) = 1;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic partition'));

PAUSE

REM
REM Cleanup
REM

DROP TABLE t PURGE;
